#include <Arduino.h>
#include <TM1637Display.h>
#include "MHZ19.h"                                        
#include <SoftwareSerial.h>
#include <Servo.h>

// Module connection pins (Digital Pins)
#define CAL 4                                              // Calibration pin, when LOW more than 7 seconds
#define BTN 2                                              //Side button input
#define RED 6                                              //RGB led red
#define BLUE 3                                             //RGB led blue
#define GREEN 5                                            //RGB led green 
#define CLK A5                                             //I2C Clock
#define DIO A4                                             //I2C Data
TM1637Display display(CLK, DIO);

#define RX_PIN 10                                          // Rx pin which the MHZ19 Tx pin is attached to
#define TX_PIN 11                                          // Tx pin which the MHZ19 Rx pin is attached to
#define BAUDRATE 9600                                      // Software serial baudrate CO2 anduriga suhtlemiseks
#define pwmPin A0                                          // PWM pin andurilt pinge lugemiseks
#define Startcountdown 60000                               // 60 sekundit käivitumiseks  
#define Calibration 300000                                 // 5 minutit kalibreerimiseks
#define servopin 9
Servo myservo;                                             // create servo object to control a servo



MHZ19 myMHZ19;                                             // Constructor for library
SoftwareSerial mySerial(RX_PIN, TX_PIN); 
const uint8_t SEG_CAL[] = {
  SEG_A | SEG_F | SEG_E | SEG_D,                   // C
  SEG_A | SEG_F | SEG_G | SEG_B | SEG_E | SEG_C,   // A
  SEG_F | SEG_E | SEG_D};                          // L
const uint8_t done[] = {
  SEG_B | SEG_C | SEG_D | SEG_E | SEG_G,           // d
  SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F,   // O
  SEG_C | SEG_E | SEG_G,                           // n
  SEG_A | SEG_D | SEG_E | SEG_F | SEG_G            // E
};
unsigned long getDataTimer = 0;
unsigned long sendDataTimer = 0;
unsigned long displayTimer = 0;
unsigned int calibrationTimer = 0;
int8_t Temp;
int RawCO2 = 400;
int CO2 = 400; 
bool button = 1; 
int offset = 0;                                            // Palju alla 400 näitab
int calMin = 0;                                            // Miinimumväärtus kalibreerimise jooksul
int lastCO2 = 400;                                         // Viimane väärtus enne praegust lugemit
bool cal = 0;                                              // Kas kalibreerimine käib või mitte (0 = ei) ja (1 = jah)
bool flag = true;                                         // Vaatame kas nuppu ei hoita all samal ajal kui Seg30() jookseb. Kui hoitakse siis false ja while loop Seg30 lõppeb ja minnakse outdoor_cal() funksiooni, muidu on alati true.
int minuteDisplay = 5; 
int val = 180;                                            // Servo asend 90 lahti / 180 kinni


void LED()                                                 // Zones: Green(400-750), Yellow(751-1150), Red(1151-5000)
{                                               
  if(400 <= CO2 && 750 >= CO2)  
  {
     analogWrite(RED, 0); 
     analogWrite(GREEN, 0);  
     analogWrite(BLUE, 1); 
      
     if(val != 180)
     {
       val = 180;
       myservo.write(val);                  // sets the servo position according to the scaled value
       delay(500);
     }
  }
  else if(751 <= CO2 && 1150 >= CO2)  
  {
     analogWrite(RED, 1); 
     analogWrite(GREEN, 0); 
     analogWrite(BLUE, 1); 

     if(val != 90)
     {
       val = 90;
       myservo.write(val);                  // sets the servo position according to the scaled value
       delay(500);
     }
  }
  else
  {
     analogWrite(RED, 1); 
     analogWrite(GREEN, 0); 
     analogWrite(BLUE, 0); 

     if(val != 90)
     {
       val = 90;
       myservo.write(val);                  // sets the servo position according to the scaled value
       delay(500);
     }
  }  
}

void Countdown()    // Laseb anduril käivituda/stabiliseeruda ja loeb nt 60 sekundi pealt alla numbreid
{                   
   display.setBrightness(0);
   display.clear();
   display.showNumberDec(((Startcountdown-millis())/1000),false, 2, 1);
   delay(998);
}

void Button()      // Funktsioon nupu lugemiseks
{                       
   button = digitalRead(BTN);
   Serial.println(button);
}

void SoftCal()      // Funktsioon anduri väärtuse kalibreerimiseks (Kui väärtus peaks langema alla 400)
{
  Serial.println(cal);
  //Serial.println(calibrationTimer);      
  if(cal == 0)
  {
    calibrationTimer = millis();     
    cal = 1;
    Serial.println(cal);
               
  }
  Serial.println("___________________Start check: lastCO2 <= 400 && (calibrationTimer - millis()) <= Calibration_______________________");
  Serial.println(cal);
  Serial.println(lastCO2);
  Serial.println(calibrationTimer - millis());
  Serial.println(Calibration);
  Serial.println("_____________________END_______________");
  if(lastCO2 <= 400 && (calibrationTimer - millis()) <= Calibration)
  { 
    if(RawCO2 < calMin)
    {
      calMin = RawCO2;
    }
    lastCO2 = RawCO2;
    Serial.println("CAL");
  }
  else if(lastCO2 <= 400 && (calibrationTimer - millis()) >= Calibration)
  { 
    if(RawCO2 < calMin)
    {
      calMin = RawCO2;
    }
    offset = (400-calMin);
    lastCO2 = RawCO2;
    Serial.println("CAL DONE");
  }
  else
  {
    cal = 0;
  }
  
}

void Carbon()
{
    RawCO2 = myMHZ19.getCO2();                             // Request CO2 (as ppm)
    
    Serial.print("Raw CO2 (ppm): ");                      
    Serial.println(RawCO2);

    CO2 = RawCO2 + offset;

    if(CO2 < 400)
    {
       CO2 = 400;  
    }
    if(CO2 > 5000)
    {
       CO2 = 5000;  
    }
        
    Temp = myMHZ19.getTemperature();                     // Request Temperature (as Celsius)
    Serial.print("Temperature (C): ");                  
    Serial.println(Temp);    
}

void seg() 
{
  //CO2= random(9999);
  Serial.println(CO2);
  display.setBrightness(0);
  display.clear();
  display.showNumberDec((CO2),false);
  //delay(5000);
}

void seg30() 
{
  displayTimer = millis();
  flag = true;
  Serial.println("__________________________________seg start___________________________________");
  
  while(millis() - displayTimer <= 30000 && flag)
  {
    Button();
    Carbon();
    seg();
    LED();
    delay(4000);
    Button();
    if(button == 0){
      flag = false;
    }
  }
  if (millis() - displayTimer <= 30000 && button == 0)
  {
    Serial.println("__________________________________outdoor start___________________________________");
    outdoor_cal();  
  }
  display.clear();
}

void outdoor_cal(){
  displayTimer = millis();
  minuteDisplay = 5;
  while(millis() - displayTimer <= 300000){
    // kalibreerimine CO2 jaoks
    display.setBrightness(0);
    display.clear();
    display.showNumberDec((((displayTimer+300000 - millis())/60000) + 1),false);
    display.setSegments(SEG_CAL, 3, 0);
    delay(5000);
  }
  pinMode(CAL, OUTPUT);
  digitalWrite(CAL, 0);

  display.setBrightness(0);
  display.clear();
  display.setSegments(done);
  
  delay(8000);
  pinMode(CAL, INPUT);
  Carbon();
  button = 1;
  seg30();
}
  

void setup() {
  
  Serial.begin(9600); 
  myservo.attach(9);  // attaches the servo on pin 9 to the servo object 
  pinMode(CAL, INPUT);
  pinMode(pwmPin, INPUT);
  display.clear();
  mySerial.begin(BAUDRATE);                               // (Uno example) device to MH-Z19 Serial start   
  myMHZ19.begin(mySerial);                                // *Serial(Stream) refence must be passed to library begin(). 
  myMHZ19.autoCalibration(false);
  pinMode(RED, OUTPUT);
  pinMode(BLUE, OUTPUT);
  pinMode(GREEN, OUTPUT);
  pinMode(BTN, INPUT_PULLUP);
  
  while(millis()< 60000)
  {
    Countdown();
  }
  
  seg30();
}

void loop() 
{
  Serial.println("-----------");
  
  if (millis() - getDataTimer >= 6000)
  {
    Carbon();
    getDataTimer = millis();
  }
  
  if(RawCO2 <= 400)
  {
    SoftCal();
  }
  
  LED();
  Button();
  if (button == 0)
  {
    seg30();
  }
  //delay(200);
}
